using System;
using System.Collections;
using System.Collections.Generic;

namespace Geno {
	enum Suits { Hearts, Diamonds, Clubs, Spades };

	class Card {
		public int Val;
		public Suits Suit;
	}

	class Player {
		public List<Card> Cards = new List<Card>();
		public int Score;
	}

	class Geno {
		List<Card> Cards = new List<Card>();
		List<Player> Players = new List<Player>();

		Random Rand = new Random();

		static void Main(string[] args) {
			Geno game = new Geno();

			game.Play();
		}

		void Play() {
			foreach (Suits suit in Enum.GetValues(typeof(Suits))) {
				for (int i = 1; i < 14; ++i) {
					Card card = new Card();
					card.Val = i;
					card.Suit = suit;

					// skip adding a queen, so the queen of hearts is by herself
					if (card.Val == 12 && card.Suit == Suits.Diamonds) continue;

					Cards.Add(card);
				}
			}

			for (int i = 0; i < 4; ++i) {
				Player player = new Player();
				Players.Add(player);
			}

			ShuffleCards();

			// deal out all the cards to the players
			int playernum = 0;

			while (Cards.Count > 0) {
				Players[playernum].Cards.Add(Cards[0]);
				Cards.RemoveAt(0);
				++playernum;
				if (playernum == Players.Count) playernum = 0;
			}

			// remove all initial pairs
			for (int i = 0; i < Players.Count; ++i) {
				RemovePlayerPairs(i);
			}

			// now make players trade one card at a time to make pairs, until someone loses

			bool finished = false;

			while (!finished) {
				int playersleft = Players.Count;

				for (int i = 0; i < Players.Count; ++i) {
					Player player = Players[i];

					if (player.Cards.Count == 0) {
						// this player is out - skip 'em!
						--playersleft;
						continue;
					}

					int playertouse = -1;

					for (int j = i + 1; j < Players.Count; ++j) {
						if (Players[j].Cards.Count > 0) {
							playertouse = j;
							break;
						}
					}

					// didn't find a player above us; try at the beginning
					if (playertouse == -1) {
						for (int j = 0; j < Players.Count; ++j) {
							if (Players[j].Cards.Count > 0) {
								playertouse = j;
								break;
							}
						}
					}

					// sanity check - have we just picked ourself?
					if (playertouse == i) {
						// yes; we're done here!
						finished = true;
						break;
					} else {
						// got a legit player - switch the card over
						int cardtochoose = Rand.Next(0, Players[playertouse].Cards.Count);
						Players[i].Cards.Add(Players[playertouse].Cards[cardtochoose]);
						Players[playertouse].Cards.RemoveAt(cardtochoose);

						// now re-search for pairs for this player
						RemovePlayerPairs(i);
					}


				}

				if (playersleft == 1) finished = true;
			}

			PrintResults();
		}

		void ShuffleCards() {
			for (int i = 0; i < Cards.Count; ++i) {
				Card tmp = Cards[i];
				Cards.RemoveAt(i);
				Cards.Insert(Rand.Next(0, Cards.Count), tmp);
			}
		}

		void RemovePlayerPairs(int player) {
			while (TryPairRemove(player)) {
				++Players[player].Score;
			}
		}

		bool TryPairRemove(int player) {
			Card card1;
			Card card2;
			Player thisplayer = Players[player];

			for (int i = 0; i < thisplayer.Cards.Count; ++i) {
				card1 = thisplayer.Cards[i];

				// can't make a pair with the Queen of Hearts!
				if (card1.Suit == Suits.Hearts && card1.Val == 12) continue;

				for (int j = i + 1; j < thisplayer.Cards.Count; ++j) {
					card2 = thisplayer.Cards[j];

					// can't make a pair with the Queen of Hearts!
					if (card2.Suit == Suits.Hearts && card2.Val == 12) continue;

					if (card1.Val == card2.Val) {
						thisplayer.Cards.RemoveAt(j);
						thisplayer.Cards.RemoveAt(i);

						Console.WriteLine("Player " + (player + 1) + " plays " + CardName(card1) + " and " + CardName(card2));

						return true;
					}
				}
			}

			return false;
		}

		string CardName(Card card) {
			string cardname = "";

			switch (card.Val) {
				case 1:
					cardname += "Ace of ";
					break;
				case 11:
					cardname += "Jack of ";
					break;
				case 12:
					cardname += "Queen of ";
					break;
				case 13:
					cardname += "King of ";
					break;
				default:
					cardname += card.Val + " of ";
					break;
			}

			cardname += Enum.GetName(typeof(Suits), card.Suit);
			return cardname;
		}


		void PrintResults() {
			Console.WriteLine("");

			for (int i = 0; i < Players.Count; ++i) {
				if (Players[i].Cards.Count > 0) {
					Console.WriteLine("Player " + (i + 1) + " has the Queen of Hearts!");
					break;
				}
			}
		}
	}
}
